import dataclasses
from typing import Any, Optional

import pyckish.types.http_codes as status


@dataclasses.dataclass
class HTTPResponse:
    body: Any = None
    headers: Optional[dict] = None
    status_code: Optional[int] = None

    def __call__(self) -> dict:
        response_dict = {}
        if self.body:
            response_dict['Body'] = self.body
        if self.headers:
            if 'Content-Type' not in self.headers.keys():
                self.headers['Content-Type'] = 'application/json'
        else:
            self.headers = {'Content-Type': 'application/json'}
        response_dict['Headers'] = self.headers
        response_dict['StatusCode'] = self.status_code if self.status_code else status.HTTP_201_CREATED
        return response_dict



from pyckish.basic_elements.context import Context
from pyckish.basic_elements.context_element import ContextElement
from pyckish.basic_elements.event import Event
from pyckish.basic_elements.event_element import EventElement



import dataclasses
import inspect
from abc import ABC, abstractmethod
from typing import Optional, Any

EMPTY = inspect.Parameter.empty


@dataclasses.dataclass
class LambdaInput:
    event: dict
    context: dict


class LambdaInputElement(ABC):
    """
    Your custom Event Element needs to be a child of this class
    """

    def __init__(self, alias: Optional[str] = None, default: Any = EMPTY()) -> None:
        self.alias = alias
        self.default = default
        self.parameter_name: Optional[str] = None

    @abstractmethod
    def extract(self, lambda_input: LambdaInput) -> Any:
        ...



from typing import Optional, Any

from pyckish.exceptions.validation_error import ValidationError
from pyckish.lambda_input_element import LambdaInputElement, LambdaInput


class PathParameter(LambdaInputElement):
    """
    Use this class on your Lambda Handler Function.
    Extracts a single HTTP Path Parameter
    """

    def __init__(self, alias: Optional[str] = None) -> None:
        super().__init__(alias=alias)

    def extract(self, lambda_input: LambdaInput) -> Any:
        key = self.alias if self.alias is not None else self.parameter_name
        try:
            argument = lambda_input.event['pathParameters'][key]
        except (KeyError, AttributeError):
            raise ValidationError(f'Path Parameter: "{key}" could not be found in event')
        return argument



from pyckish import LambdaInputElement
from pyckish.lambda_input_element import LambdaInput


class Event(LambdaInputElement):
    """
    Use this class on your Lambda Handler Function.
    It just returns the raw event.
    """

    def extract(self, lambda_input: LambdaInput) -> dict:
        return lambda_input.event



from typing import Optional, Any

from pyckish import LambdaInputElement, EMPTY
from pyckish.exceptions.validation_error import ValidationError
from pyckish.lambda_input_element import LambdaInput


class EventElement(LambdaInputElement):
    """
    Use this class on your Lambda Handler Function.
    Extracts a single key from the AWS Event
    """

    def __init__(self, alias: Optional[str] = None, default: Any = EMPTY()) -> None:
        super().__init__(alias=alias, default=default)

    def extract(self, lambda_input: LambdaInput) -> Any:
        key = self.alias if self.alias is not None else self.parameter_name
        try:
            return lambda_input.event[key]
        except (KeyError, AttributeError):
            if type(self.default) == EMPTY:
                raise ValidationError(f'The event parameter "{key}" could not be found')
            return self.default
